/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _CENTRAL_TOOLBAR_H_
#define _CENTRAL_TOOLBAR_H_

#include "viewmode.h"

#include <QToolBar>
#include <QList>
#include <QModelIndex>

#include <QToolButton>

#include <QSignalMapper>
#include <QActionGroup>
#include <QAction>
#include <QtCore>

class LineEditClear;
class NavigatorBar; 

/*
********************************************************************************
*                                                                              *
*    Class CentralToolBar                                                      *
*                                                                              *
********************************************************************************
*/
class CentralToolBar : public QToolBar
{
Q_OBJECT
    static CentralToolBar* INSTANCE;
  public:
    CentralToolBar(QWidget *parent);
    static CentralToolBar* instance() { return INSTANCE; }
    
    void updateMode(MAIN::ViewMode mode);
    void updateCurrentIndex(const QModelIndex &idx);

    void set_enable_jump_to(bool b);
    
  private slots:
    void slot_send_search();
    void slot_histo_prev_activated();
    void slot_histo_next_activated();
     
  signals:
    void searchActivated(const QString);
    void prevNextActivated(QModelIndex, QString);
    void dbNameChanged();

  private:
    struct HistoryParam
    {
      HistoryParam() {}
      HistoryParam(const QModelIndex &i,const QString& text)
      {
        idx = i;
        search_pattern = text;      
      }
      
      QModelIndex  idx;
      QString      search_pattern;
    };
      
    LineEditClear        *ui_line_edit;
    NavigatorBar         *m_navigatorBar;
    QToolButton          *ui_goto_button;    
    
    //! History prev/next navigation
    QList<HistoryParam>   m_history_params;
    int                   m_history_current_idx;

};

/*
********************************************************************************
*                                                                              *
*    Class MainToolButton                                                      *
*                                                                              *
********************************************************************************
*/
class MainToolButton : public QToolButton
{
 Q_OBJECT
  public:
    MainToolButton(QWidget *parent);
    
  private:
    enum Q_MODE {MODE_TITLE = 1,MODE_ALBUM = 2,MODE_ARTIST = 3, MODE_EXTENDED=4};

    void addMenuItem(QSignalMapper* mapper, QActionGroup* group,QAction* action, Q_MODE mode);
    void updateMultiDbMenu();    
  
  private slots:
     void slot_dbNameClicked();
     void slot_showMenu();

  signals:
    void dbNameChanged();
      
  private:
    QMenu               *m_menu;
    QMenu               *m_menuChooseDb;
    QAction             *m_menuChooseDbAction;
};



#endif // _CENTRAL_TOOLBAR_H_
