/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CentralWidget/menumodel.h"
#include "CentralWidget/menuitem.h"
#include "viewmode.h"
#include "utilities.h"      // CONFIGDIR
#include "global_actions.h"
#include "debug.h"


#include <QDir>
#include <QMap>
#include <QString>
#include <QStringList>
#include <QAction>


MenuModel* MenuModel::INSTANCE = 0;

/*
********************************************************************************
*                                                                              *
*    Class MenuModel                                                           *
*                                                                              *
********************************************************************************
*/
MenuModel::MenuModel(QObject *parent) : QStandardItemModel(parent)
{
    INSTANCE = this;
    configureGlobalAction();
    populateMenu(); 
    configureMenuAction(); 
}


MenuModel::~MenuModel()
{
    delete m_radioItem;
}

/*******************************************************************************
    configureGlobalAction
*******************************************************************************/
void MenuModel::configureGlobalAction()
{
    QMap<ENUM_ACTION, QAction*> *actions = ACTIONS(); 

    actions->insert(BROWSER_VIEW_SETTINGS, new QAction(this));
    actions->insert(BROWSER_VIEW_DASHBOARD, new QAction(this));
    actions->insert(BROWSER_VIEW_CONTEXT, new QAction(this));
    actions->insert(BROWSER_VIEW_HISTO, new QAction(this));
    actions->insert(BROWSER_VIEW_ALBUM,new QAction(this));
    actions->insert(BROWSER_VIEW_ARTIST,  new QAction(this));
    actions->insert(BROWSER_VIEW_TRACK, new QAction(this));
    actions->insert(BROWSER_VIEW_GENRE, new QAction(this));
    actions->insert(BROWSER_VIEW_YEAR, new QAction(this));
    actions->insert(BROWSER_VIEW_FAVORITE, new QAction(this));
    actions->insert(BROWSER_VIEW_PLAYLIST, new QAction(this));
    actions->insert(BROWSER_VIEW_SMART_PLAYLIST, new QAction(this));
    actions->insert(BROWSER_VIEW_ICECAST, new QAction(this));
    actions->insert(BROWSER_VIEW_SHOUTCAST, new QAction(this));
    actions->insert(BROWSER_VIEW_RADIO, new QAction(this));
    actions->insert(BROWSER_VIEW_FAVORITE_RADIO, new QAction(this));

    //! pour chaque actions on utilise le ViewMode (cf. viewmode.h)
    actions->value(BROWSER_VIEW_SETTINGS)->setData(QVariant(int(MAIN::ViewSettings)));
    actions->value(BROWSER_VIEW_DASHBOARD)->setData(QVariant(int(MAIN::ViewDashBoard)));
    actions->value(BROWSER_VIEW_CONTEXT)->setData(QVariant(int(MAIN::ViewContext)));
    actions->value(BROWSER_VIEW_HISTO)->setData(QVariant(int(MAIN::ViewHistory)));
    actions->value(BROWSER_VIEW_ALBUM)->setData(QVariant(int(MAIN::ViewAlbum)));
    actions->value(BROWSER_VIEW_ARTIST)->setData(QVariant(int(MAIN::ViewArtist)));
    actions->value(BROWSER_VIEW_TRACK)->setData(QVariant(int(MAIN::ViewTrack)));
    actions->value(BROWSER_VIEW_GENRE)->setData(QVariant(int(MAIN::ViewGenre)));
    actions->value(BROWSER_VIEW_YEAR)->setData(QVariant(int(MAIN::ViewYear)));
    actions->value(BROWSER_VIEW_FAVORITE)->setData(QVariant(int(MAIN::ViewFavorite)));
    actions->value(BROWSER_VIEW_PLAYLIST)->setData(QVariant(int(MAIN::ViewPlaylist)));
    actions->value(BROWSER_VIEW_SMART_PLAYLIST)->setData(QVariant(int(MAIN::ViewSmartPlaylist)));
    actions->value(BROWSER_VIEW_ICECAST)->setData(QVariant(int(MAIN::ViewIceCast)));
    actions->value(BROWSER_VIEW_SHOUTCAST)->setData(QVariant(int(MAIN::ViewShoutCast)));
    actions->value(BROWSER_VIEW_RADIO)->setData(QVariant(int(MAIN::ViewCustomRadio)));
    actions->value(BROWSER_VIEW_FAVORITE_RADIO)->setData(QVariant(int(MAIN::ViewFavoriteRadio)));
}

/*******************************************************************************
    populateMenu
*******************************************************************************/
void MenuModel::populateMenu()
{
   QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
    
   QStandardItem *rootItem = this->invisibleRootItem();

   //! populate Root Item
   MenuItem* item0 = new MenuItem(tr("Settings"), QIcon(":/images/settings.png"));
   item0->setData(QVariant::fromValue(new QAction(QIcon(":/images/settings.png"), tr("Settings"), this)), MenuActionRole);
   item0->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_SETTINGS)), GlobalActionRole);
   rootItem->appendRow(item0);
   
   MenuItem* item1 = new MenuItem(tr("Context view"), QIcon(":/images/info-48x48.png"));
   item1->setData(QVariant::fromValue(new QAction(QIcon(":/images/info-48x48.png"), tr("Context view"), this)), MenuActionRole);
   item1->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_CONTEXT)), GlobalActionRole);
   rootItem->appendRow(item1);

   MenuItem* item5 = new MenuItem(tr("Dashboard view"), QIcon(":/images/chart-64x64.png"));
   item5->setData(QVariant::fromValue(new QAction(QIcon(":/images/chart-64x64.png"), tr("Dashboard view"), this)), MenuActionRole);
   item5->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_DASHBOARD)), GlobalActionRole);
   rootItem->appendRow(item5);

   MenuItem* item4 = new MenuItem(tr("History view"), QIcon(":/images/history-48x48.png"));
   item4->setData(QVariant::fromValue(new QAction(QIcon(":/images/history-48x48.png"), tr("History view"), this)), MenuActionRole);
   item4->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_HISTO)), GlobalActionRole);
   rootItem->appendRow(item4);
   
   MenuItem* item2 = new MenuItem(tr("Music browser"), QIcon(":/images/folder-music-48x48.png"));
   item2->setData(QVariant::fromValue(new QAction(QIcon(":/images/folder-music-48x48.png"), tr("Music browser"), this)), MenuActionRole);
   rootItem->appendRow(item2);

   MenuItem* item3 = new MenuItem(tr("Playlist browser"), QIcon(":/images/media-playlist-48x48.png"));
   item3->setData(QVariant::fromValue(new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Playlist browser"), this)), MenuActionRole);
   rootItem->appendRow(item3);

   m_radioItem = new MenuItem(tr("Radio browser"), QIcon(":/images/media-url-48x48.png"));
   m_radioItem->setData(QVariant::fromValue(new QAction(QIcon(":/images/media-url-48x48.png"), tr("Radio browser"), this)), MenuActionRole);   
   rootItem->appendRow(m_radioItem);
 
   /*
   MenuItem* item6 = new MenuItem(tr("Internet services"), QIcon(":/images/internet.png"), MenuFirstLevelType);
   homeItem->appendRow(item6);
   actions->value("m_internetBrowser")->setData(QVariant::fromValue(indexFromItem(item6)));
   item6->setData(QVariant::fromValue(actions->value("m_internetBrowser")), ActionRole);
   */  
   
   //! populate Collection Item
   MenuItem* item21 = new MenuItem(tr("album view"), QIcon(":/images/album.png"));
   item21->setData(QVariant::fromValue(new QAction(QIcon(":/images/album.png"), tr("album view"), this)), MenuActionRole);
   item21->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_ALBUM)), GlobalActionRole);
   item2->appendRow(item21);

   MenuItem* item22 = new MenuItem(tr("artist view"), QIcon(":/images/view-artist.png")); 
   item22->setData(QVariant::fromValue(new QAction(QIcon(":/images/view-artist.png"), tr("artist view"), this)), MenuActionRole);
   item22->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_ARTIST)), GlobalActionRole);
   item2->appendRow(item22);

   MenuItem* item23 = new MenuItem(tr("track view"), QIcon(":/images/track-48x48.png")); 
   item23->setData(QVariant::fromValue(new QAction(QIcon(":/images/track-48x48.png"), tr("track view"), this)), MenuActionRole);
   item23->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_TRACK)), GlobalActionRole);
   item2->appendRow(item23);

   MenuItem* item24 = new MenuItem(tr("genre view"), QIcon(":/images/genre.png")); 
   item24->setData(QVariant::fromValue(new QAction(QIcon(":/images/genre.png"), tr("genre view"), this)), MenuActionRole);
   item24->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_GENRE)), GlobalActionRole);
   item2->appendRow(item24);
   
   MenuItem* item25 = new MenuItem(tr("year view"), QIcon(":/images/date-48x48.png")); 
   item25->setData(QVariant::fromValue(new QAction(QIcon(":/images/date-48x48.png"), tr("year view"), this)), MenuActionRole);
   item25->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_YEAR)), GlobalActionRole);
   item2->appendRow(item25);
   
   MenuItem* item26 = new MenuItem(tr("favorite view"), QIcon(":/images/favorites-48x48.png")); 
   item26->setData(QVariant::fromValue(new QAction(QIcon(":/images/favorites-48x48.png"), tr("favorite view"), this)), MenuActionRole);
   item26->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_FAVORITE)), GlobalActionRole);
   item2->appendRow(item26);

   //! populate Paylist Browser Item
   MenuItem* item31 = new MenuItem(tr("playlist view"), QIcon(":/images/media-playlist-48x48.png")); 
   item31->setData(QVariant::fromValue(new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("playlist view"), this)), MenuActionRole);
   item31->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_PLAYLIST)), GlobalActionRole);
   item3->appendRow(item31);

   MenuItem* item32 = new MenuItem(tr("smart playlist view"), QIcon(":/images/smart-playlist-48x48.png")); 
   item32->setData(QVariant::fromValue(new QAction(QIcon(":/images/smart-playlist-48x48.png"), tr("smart playlist view"), this)), MenuActionRole);
   item32->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_SMART_PLAYLIST)), GlobalActionRole);
   item3->appendRow(item32);

   //! populate Radio Browser Item
   populateRadioItem();
   
  /*//! populate Internet services Item
   MenuItem* item61 = new MenuItem(tr("jamendo service"), QIcon(":/images/jamendo.png"), MenuSecondLevelType); 
   item6->appendRow(item61);
   actions->value("m_jamendoview")->setData(QVariant::fromValue(indexFromItem(item61)));
   item61->setData(QVariant::fromValue(actions->value("m_jamendoview")), ActionRole);
*/
}

/*******************************************************************************
    populateRadioItem
*******************************************************************************/
void MenuModel::populateRadioItem() 
{
    QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();

    //! removeRows for radio Item
    m_radioItem->removeRows(0, m_radioItem->rowCount() ); 

    //! add IceCast
    MenuItem *mi1 = new MenuItem(tr("icecast stream"), QIcon(":/images/icecast.png"));
    mi1->setData(QVariant::fromValue(new QAction(QIcon(":/images/icecast.png"), tr("icecast stream"), this)), MenuActionRole);
    mi1->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_ICECAST)), GlobalActionRole);
    m_radioItem->appendRow(mi1);

    //! add ShoutCast
    MenuItem *mi2 = new MenuItem(tr("shoutcast stream"), QIcon(":/images/shoutcast.png"));
    mi2->setData(QVariant::fromValue(new QAction(QIcon(":/images/shoutcast.png"), tr("shoutcast stream"), this)), MenuActionRole);
    mi2->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_SHOUTCAST)), GlobalActionRole);
    m_radioItem->appendRow(mi2);
  
    //! search for xspf custom radio file 
    const QStringList filters = QStringList() << "*.xspf";
    QDir sourceDir(UTIL::CONFIGDIR + "/radio");
    sourceDir.setNameFilters(filters);  

    QStringList entryList = sourceDir.entryList(filters, QDir::Files, QDir::Name);
    foreach (const QString &file, entryList) 
    {
      //Debug::debug() << "-- MenuModel --> populateUserRadioFile : custom radio file  " << file;

      MenuItem *mi = new MenuItem(QFileInfo(file).baseName(),QIcon(":/images/media-url-48x48.png"));
  
      mi->setData(QVariant::fromValue(new QAction(QIcon(":/images/media-url-48x48.png"), QFileInfo(file).baseName(), this)), MenuActionRole);
      mi->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_RADIO)), GlobalActionRole);
      mi->setData(QVariant(file), FileRole);
     m_radioItem->appendRow(mi);
    }
    
    //! add Favorite
    MenuItem *mi3 = new MenuItem(tr("favorite stream"), QIcon(":/images/favorites-48x48.png"));
    mi3->setData(QVariant::fromValue(new QAction(QIcon(":/images/favorites-48x48.png"), tr("favorite stream"), this)), MenuActionRole);
    mi3->setData(QVariant::fromValue(actions->value(BROWSER_VIEW_FAVORITE_RADIO)), GlobalActionRole);
    m_radioItem->appendRow(mi3);    
}

/*******************************************************************************
    configureMenuAction
*******************************************************************************/
void MenuModel::configureMenuAction() 
{
   for (int i=0; i < this->rowCount(QModelIndex()); i++) 
   {
    const QModelIndex childIdx = this->index(i, 0, QModelIndex());
    QAction *a = childIdx.data(MenuActionRole).value<QAction*>();
    a->setData(QVariant::fromValue(childIdx));
    connect(a, SIGNAL(triggered()),this, SLOT(findSender()));
  
    QStandardItem *childItem = itemFromIndex( childIdx );
  
    for (int j=0; j < childItem->rowCount(); j++) {
         const QModelIndex child2Idx = this->index(j, 0, childIdx);
         QAction *a = child2Idx.data(MenuActionRole).value<QAction*>();
         a->setData(QVariant::fromValue(child2Idx));
         connect(a, SIGNAL(triggered()),this, SLOT(findSender()));
      }
  }
}

/*******************************************************************************
    update
     -> mise  jour ncessaire lors d'un ajout de fichier de radio utilisateur
*******************************************************************************/
void MenuModel::update()
{ 
    populateRadioItem();
    configureMenuAction();
    emit modelUpdated();
} 

/*******************************************************************************
    restoreModelIdx
*******************************************************************************/
QModelIndex MenuModel::restoreModelIdx(MAIN::ViewMode m, const QString& file)
{
  //Debug::debug() << "-- MenuModel::IndexFromViewMode rechercher ViewMode : " << m;

  for (int j=0; j < this->rowCount(QModelIndex()); j++) 
  {
    const QModelIndex childIdx = this->index(j, 0, QModelIndex());
      
    QAction *a = childIdx.data(GlobalActionRole).value<QAction*>();
    if(a) {
      MAIN::ViewMode mode = MAIN::ViewMode(a->data().toInt());
      if(mode == m)
        return childIdx;
    }

    for (int i=0; i < this->rowCount(childIdx); i++) 
    {
        const QModelIndex child2Idx = this->index(i, 0, childIdx);

        QAction *a = child2Idx.data(GlobalActionRole).value<QAction*>();
        if(a) {
          MAIN::ViewMode mode = MAIN::ViewMode(a->data().toInt());

          if(mode == m) {
              if(m != MAIN::ViewCustomRadio) { 
                return child2Idx;
              }
              else {
                 if( child2Idx.data(FileRole).toString() == file )
                   return child2Idx;
              }
          }
        }
    }
  }

  return QModelIndex();
}


/*******************************************************************************
    findSender
     -> Quand une action provenant du menu (action associ au item et visible 
     par les menu de navigatorbar --> on retrouve l'index dans le modele
*******************************************************************************/
void MenuModel::findSender()
{
   QAction *action = qobject_cast<QAction *>(sender());
   if(!action) return;
   
   QModelIndex idx = action->data().value<QModelIndex>();
    
   //Debug::debug() << "-- MenuModel --> findSender idx" << idx;
   emit modelItemActivated(idx);
}

/*******************************************************************************
    activate_view_idx
*******************************************************************************/
void MenuModel::activate_view_idx(MAIN::ViewMode m)
{
    QModelIndex idx = restoreModelIdx(m, QString() );

    if(idx.isValid())
      emit modelItemActivated(idx);
}

