/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CollectionView/graphicitem.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"
#include "utilities.h"

#include "qpixmapfilter.h"                        // 0.0.44

#include "CollectionModel/collectionmodel.h"      // 0.0.45
#include "CollectionModel/plsfilemodel.h"         // 0.0.45

#include "SmartPlaylist/smartplaylist.h"          // 0.0.49

#include "Widget/ratingwidget.h"                  // 0.0.55
#include "global_actions.h"

#include "debug.h"


//! Qt
#include <QPen>
#include <QStyle>



void draw_playcount(QPainter* painter, int Xstart,int Ystart, int Xstop);

/*
********************************************************************************
*                                                                              *
*    InfoGraphicItem                                                           *
*                                                                              * 
********************************************************************************
*/
InfoGraphicItem::InfoGraphicItem() 
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   _width = 630;
}

QRectF InfoGraphicItem::boundingRect() const
{
    return QRectF(0, 0, _width, 40);
}

void InfoGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
   const QRect rect = boundingRect().toRect();
   
   painter->setFont(QFont("Segoe UI", 10, QFont::Normal));
   painter->setPen(QColor(0x555555));    

   QFontMetrics metric( painter->font() );
   const QString elided = metric.elidedText ( _text, Qt::ElideRight, _width -100);
    
   painter->drawPixmap(rect.adjusted(10,-4,0,0).topLeft() ,QPixmap(":/images/info-message.png") );   
   painter->drawText(rect.adjusted(70,6,-100,-10), Qt::AlignVCenter, elided);
}

/*
********************************************************************************
*                                                                              *
*    CategorieGraphicItem                                                      *
*                                                                              * 
********************************************************************************
*/
CategorieGraphicItem::CategorieGraphicItem() 
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   _width = 630;
}

QRectF CategorieGraphicItem::boundingRect() const
{
    return QRectF(0, 0, _width, 50);
}

void CategorieGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)

   //! draw categorie text
  //QFontDatabase * fontdb = new QFontDatabase();
  //painter->setFont(fontdb->font("Segoe UI", "Bold", 14) );
  painter->setFont( QFont("Segoe UI", 14, QFont::Bold) );
  
  QColor inlineColor = QColor(0xfca822);
  QColor outlineColor = QColor (inlineColor.lighter(140) );

  painter->setPen(inlineColor);
  painter->drawStaticText(5,0, QStaticText(_catName));
 
  //BEGIN: horizontal line
  {
    QPoint start(boundingRect().toRect().left()+3, boundingRect().toRect().top()+22);
    start.rx() += 3;
    QPoint horizontalGradTop(boundingRect().toRect().topLeft());
    horizontalGradTop.rx() += boundingRect().toRect().width() - 6;
    QLinearGradient gradient(start, horizontalGradTop);
    gradient.setColorAt(0, inlineColor);
    gradient.setColorAt(1, outlineColor);	
    painter->fillRect(QRect(start, QSize(boundingRect().toRect().width() - 6, 1)), gradient);
  }
  //END: horizontal line
}

/*
********************************************************************************
*                                                                              *
*    AlbumGraphicItem                                                          *
*                                                                              * 
********************************************************************************
*/
AlbumGraphicItem::AlbumGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, true);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
   opt.state &= ~QStyle::State_Selected;
}


QRectF AlbumGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 150, 150);
}

void AlbumGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get Model data 
   const QString _title     =  media->data.at(ALBUM::name).toString();
   const QString _coverPath =  media->data.at(ALBUM::coverpath).toString();
   const QString _artist    =  media->parent() ? media->parent()->data.at(ARTIST::name).toString() : "";
   
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
   
   QPixmap pixTemp(QSize(150,150));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
 
      //! paint cover art
      QPixmap pix;
      if(!QPixmapCache::find(_coverPath, pix)) {
         pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPath);
         if(pix.isNull()) {
            pix = QPixmap(":/images/emptycoverblue110x110.png");
         }
         else {
           QPixmapCache::insert(_coverPath, pix);
         }
      }
      
      p.drawPixmap(20,2, pix);

    p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
   
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),pixTemp,boundingRect());  

   //! paint album title
   painter->setPen(QColor("black"));
   painter->setFont(opt.font);
    
   QFontMetrics m1( painter->font());
   const QString elided_album = m1.elidedText ( _title, Qt::ElideRight, 150);
   painter->drawText(QRect (0,110,150, 25), Qt::AlignVCenter | Qt::AlignCenter,elided_album );
   
   //! paint artist name
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->setPen(QColor(0x555555));
   
   QFontMetrics m2(painter->font());
   const QString elided_artist = m2.elidedText ( _artist, Qt::ElideRight, 150, Qt::TextWrapAnywhere);   
   painter->drawText(QRect(0, 132, 150, 18), Qt::AlignTop | Qt::AlignCenter, elided_artist);      
   
   //! paint playing or favorite attibute
   if(media->data.at(ALBUM::isPlaying).toBool())
     painter->drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      
   if(media->data.at(ALBUM::isFavorite).toBool())
     painter->drawPixmap(121, 30, QPixmap(":/images/favorites-18x18.png"));
}




void AlbumGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  //Debug::debug() << " ---- AlbumGraphicItem::hoverEnterEvent";
  Q_UNUSED(event)
  opt.state |= QStyle::State_MouseOver;

  this->update();
}

void AlbumGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  //Debug::debug() << " ---- AlbumGraphicItem::hoverLeaveEvent";
  opt.state &= ~QStyle::State_MouseOver;  

  this->update();
}

void AlbumGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::ClosedHandCursor);  
}

void AlbumGraphicItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::OpenHandCursor);
}

void AlbumGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event) 
}

void AlbumGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->setData(v); 
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->trigger();
} 



void AlbumGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) {
      if( CollectionModel::instance()->isTrackFiltered(this->media->child(i)) )
        mimedata->addMediaItem(this->media->child(i));
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    QPixmap pix;
    QPixmapCache::find(media->data.at(ALBUM::coverpath).toString(), pix);
    if(!pix.isNull())
      drag->setPixmap(pix);
    
    drag->exec();
    setCursor(Qt::OpenHandCursor);
}

/*
********************************************************************************
*                                                                              *
*    AlbumGraphicItem_v2                                                       *
*                                                                              * 
********************************************************************************
*/
AlbumGraphicItem_v2::AlbumGraphicItem_v2()
{

}



void AlbumGraphicItem_v2::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get Model data 
   const QString _title     =  media->data.at(ALBUM::name).toString();
   const QString _year      =  media->data.at(ALBUM::year).toString();
   const QString _coverPath =  media->data.at(ALBUM::coverpath).toString();
   
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
   
   QPixmap pixTemp(QSize(150,150));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
 
      //! paint cover art
      QPixmap pix;
      if(!QPixmapCache::find(_coverPath, pix)) {
         pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPath);
         if(pix.isNull()) {
            pix = QPixmap(":/images/emptycoverblue110x110.png");
         }
         else {
           QPixmapCache::insert(_coverPath, pix);
         }
      }
      
      p.drawPixmap(20,2, pix);

    p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
   
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),pixTemp,boundingRect());  

   //! paint album title
   painter->setPen(QColor("black"));
   painter->setFont(opt.font);
    
   QFontMetrics m1( painter->font());
   const QString elided_album = m1.elidedText ( _title, Qt::ElideRight, 150);
   painter->drawText(QRect (0,110,150, 25), Qt::AlignVCenter | Qt::AlignCenter,elided_album );
   
   //! paint year
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->setPen(QColor(0x555555));
   painter->drawText(QRect(0, 132, 150, 18), Qt::AlignTop | Qt::AlignCenter, _year);      

   //! paint playing or favorite attibute
   if(media->data.at(ALBUM::isPlaying).toBool())
     painter->drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      
   if(media->data.at(ALBUM::isFavorite).toBool())
     painter->drawPixmap(121, 30, QPixmap(":/images/favorites-18x18.png"));
}
/*
********************************************************************************
*                                                                              *
*    AlbumGenreGraphicItem                                                     *
*                                                                              * 
********************************************************************************
*/
AlbumGenreGraphicItem::AlbumGenreGraphicItem()
{

}


void AlbumGenreGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) 
    {
      MediaItemPtr trackItem = this->media->child(i);
      if(!CollectionModel::instance()->isTrackFiltered(this->media->child(i)) ) continue;
      if(trackItem->data.at(TRACK::genre).toString() == _genre)
        mimedata->addMediaItem(trackItem);
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    QPixmap pix;
    QPixmapCache::find(media->data.at(ALBUM::coverpath).toString(), pix);
    if(!pix.isNull())
      drag->setPixmap(pix);
    
    drag->exec();
    setCursor(Qt::OpenHandCursor);  
}


/*
********************************************************************************
*                                                                              *
*    ArtistGraphicItem                                                         *
*                                                                              * 
********************************************************************************
*/
ArtistGraphicItem::ArtistGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, true);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);

   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
   opt.state &= ~QStyle::State_Selected;
}

QRectF ArtistGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}



void ArtistGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   const QString      _name          = media->data.at(ARTIST::name).toString();
   const QStringList  _coverPathList = media->data.at(ARTIST::coverpathlist).toStringList();

        
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);   
     
   QPixmap basePixmap = QPixmap( 140, 150 );
   {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );

     //! affichage des covers
     for (int i = 0; i <_coverPathList.size(); i++) {
         QPixmap pix;
         if(!QPixmapCache::find(_coverPathList[i], pix)) {
           pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPathList[i]);
           if(pix.isNull()) {
             pix = QPixmap(":/images/emptycoverblue110x110.png");
           }
           else {
             QPixmapCache::insert(_coverPathList[i], pix);
           }
        }
 
         pix = pix.scaled(QSize(85,85), Qt::KeepAspectRatio, Qt::SmoothTransformation);
         qreal angle =  15*i;

         QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
         pix = pix.transformed(transform, Qt::SmoothTransformation); 
      
         pt.drawPixmap(boundingRect().toRect().adjusted((140 -pix.width())/2,
                                                        (150 -pix.height())/2 -15,
                                                       -(140 -pix.width())/2, 
                                                       -(150-pix.height())/2 -15)
                                                        , pix);
     }
     pt.end();
   }
   painter->drawPixmap(boundingRect().toRect(), basePixmap);
  
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),basePixmap,boundingRect());  
  
   //! affichage nom de l'artist
   QColor color = QColor(0x2e2b2b);
   
   painter->setPen(color);
   painter->setFont(opt.font);
    
   QFontMetrics fmetric(painter->font());
   //QRect fontrect = fmetric.boundingRect (QRect(0, 120, 140, 30),  Qt::AlignCenter | Qt::TextWrapAnywhere, _name, 0, 0 );
   const QString elided_artist = fmetric.elidedText ( _name, Qt::ElideMiddle, 260, Qt::TextWrapAnywhere);   

   painter->drawText(QRect(0, 120, 140, 30), Qt::AlignTop | Qt::AlignCenter | Qt::TextWrapAnywhere, elided_artist);      

     
   //! affichage des attributs playing ou favorite
   if(media->data.at(ARTIST::isPlaying).toBool())
     painter->drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      
   if(media->data.at(ARTIST::isFavorite).toBool())
     painter->drawPixmap(121, 30, QPixmap(":/images/favorites-18x18.png"));
}




void ArtistGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->setData(v); 
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->trigger();  
} 


void ArtistGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  //Debug::debug() << " ---- AlbumGraphicItem::hoverEnterEvent";
Q_UNUSED(event)
  opt.state |= QStyle::State_MouseOver;

  this->update();
}

void ArtistGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
Q_UNUSED(event)
  //Debug::debug() << " ---- AlbumGraphicItem::hoverLeaveEvent";
  opt.state &= ~QStyle::State_MouseOver;  

  this->update();
}

void ArtistGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::ClosedHandCursor);  
}

void ArtistGraphicItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::OpenHandCursor);
}

void ArtistGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
Q_UNUSED(event)
}

void ArtistGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    
    for(int i =0; i < this->media->childCount();i++) {
      MediaItemPtr albumItem = media->child(i);
      for(int j =0; j < albumItem->childCount();j++) 
      {
        if(!CollectionModel::instance()->isTrackFiltered(albumItem->child(j)) ) continue;
        mimedata->addMediaItem(albumItem->child(j));
      }
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    QPixmap basePixmap = QPixmap( 140, 150 );
    {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );
     for (int i = 0; i < media->data.at(ARTIST::coverpathlist).toStringList().size(); i++) 
     {
       QPixmap pix;
       QPixmapCache::find(media->data.at(ARTIST::coverpathlist).toStringList()[i], pix);
       if(pix.isNull()) continue;
       
       pix = pix.scaled(QSize(85,85), Qt::KeepAspectRatio, Qt::SmoothTransformation);
       qreal angle =  15*i;
       
       QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
       pix = pix.transformed(transform, Qt::SmoothTransformation);
       
       pt.drawPixmap(boundingRect().toRect().adjusted((140 -pix.width())/2,
                                                      (150 -pix.height())/2 -15,
                                                     -(140 -pix.width())/2, 
                                                     -(150-pix.height())/2 -15)
                                                     , pix);
     }
    }
    drag->setPixmap(basePixmap);
    
    drag->exec();  
    setCursor(Qt::OpenHandCursor);
}

/*
********************************************************************************
*                                                                              *
*    Class TrackGraphicItem                                                    *
*      -> used in PLAYLIST VIEW                                                *
*      -> draw only number/track name                                          *
********************************************************************************
*/
TrackGraphicItem::TrackGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, true);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);  
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_ListViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   
   opt.decorationPosition = QStyleOptionViewItem::Left;
   opt.decorationAlignment = Qt::AlignCenter;
   opt.displayAlignment = Qt::AlignLeft|Qt::AlignVCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;   
   
    _width = 510;
}

QRectF TrackGraphicItem::boundingRect() const
{
    if(_width < 510)
      return QRectF(0, 0, 510, 22);
    
    return QRectF(0, 0, _width, 22);
}

void TrackGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   bool isTrack       = (media->type == TYPE_TRACK) ? true : false;
   bool isPlaying     = isTrack ? media->data.at(TRACK::isPlaying).toBool() : media->data.at(STREAM::isPlaying).toBool();
   bool isBroken      = isTrack ? media->data.at(TRACK::isBroken).toBool() :  media->data.at(STREAM::isBroken).toBool();
   
   const int width       = _width < 530 ? 530 : _width;
   const int title_width = _width < 530 ? 150 : (_width - 140)/3;
   
   QPixmap pixTemp(QSize(width,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect().adjusted(0,0,-10,0);
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      //! TRACK in collection
      if(media->data.at(TRACK::id) != -1) {
        p.setPen(QColor(0x2e2b2b));
        p.setFont(opt.font);
        QFontMetrics metric( p.font());
      
        //! paint artist name
        const QString artist_elided = metric.elidedText ( media->data.at(TRACK::artist).toString(), Qt::ElideRight, title_width);
        p.drawText(QRect(30, 0, title_width, 22), Qt::AlignLeft | Qt::AlignVCenter,artist_elided);

        //! paint album name
        const QString album_elided = metric.elidedText ( media->data.at(TRACK::album).toString(), Qt::ElideRight, title_width);
        p.drawText(QRect(60+title_width, 0, title_width, 22), Qt::AlignLeft | Qt::AlignVCenter,album_elided);

        //! paint track name
        const QString title_elided = metric.elidedText ( media->data.at(TRACK::title).toString(), Qt::ElideRight, title_width);
        p.drawText(QRect(60+title_width*2, 0, title_width, 22), Qt::AlignLeft | Qt::AlignVCenter,title_elided);

        //! paint track duration
        const QString duree_elided = metric.elidedText ( MEDIA::durationTextFromInt(media->data.at(TRACK::duration).toInt()), Qt::ElideRight, 50);
        p.drawText(QRect(60+title_width*3, 0, 50, 22), Qt::AlignRight | Qt::AlignVCenter,duree_elided);
      }
      else {
        p.setPen( QColor(0x707070) );
        p.setFont(opt.font);

        QFontMetrics metric( p.font());
        const QString name_elided = metric.elidedText ( media->data.at(TRACK::url).toString(), Qt::ElideRight, width -20);
        p.drawText(QRect(30, 0,  width -20, 22), Qt::AlignLeft | Qt::AlignVCenter, name_elided);
      }
      
      //! paint activated item
      if(isPlaying)
        p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      else if(isBroken)
        p.drawPixmap(2, 1, QPixmap(":/images/media-broken-18x18.png"));
      else if (!isTrack)
        p.drawPixmap(0, 1, QPixmap(":/images/media-url-18x18.png"));
      
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);   
}


void TrackGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->setData(v); 
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->trigger();  
} 


void TrackGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  //Debug::debug() << " ---- AlbumGraphicItem::hoverEnterEvent";
Q_UNUSED(event)
  opt.state |= QStyle::State_MouseOver;

  this->update();
}

void TrackGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
Q_UNUSED(event)
  //Debug::debug() << " ---- AlbumGraphicItem::hoverLeaveEvent";
  opt.state &= ~QStyle::State_MouseOver;  

  this->update();
}

void TrackGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::ClosedHandCursor);  
}

void TrackGraphicItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::OpenHandCursor);
}

void TrackGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
Q_UNUSED(event)
}



void TrackGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    mimedata->addMediaItem(this->media);
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    drag->exec();
    setCursor(Qt::OpenHandCursor);    
}

/*
********************************************************************************
*                                                                              *
*    Class TrackGraphicItem_v2                                                 *
*      -> used in TRACKS VIEW                                                  *
*      -> draw number/track name/album/duration/rating                         *
*                                                                              *
********************************************************************************
*/  
TrackGraphicItem_v2::TrackGraphicItem_v2()
{
    _width = 530;
}

QRectF TrackGraphicItem_v2::boundingRect() const
{
    if(_width < 530)
      return QRectF(0, 0, 530, 22);
    
    return QRectF(0, 0, _width, 22);
}

void TrackGraphicItem_v2::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   const int width       = _width < 530 ? 530 : _width;
   const int title_width = _width < 530 ? 150 : (_width - 210)/2;
   
   if(media->type != TYPE_TRACK) {
     TrackGraphicItem::paint(painter, option, widget);
     return;
   }
   
   QPixmap pixTemp(QSize(width,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect().adjusted(0,0,-10,0);
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);      
      
      //! paint track number 
      QColor color = QColor(0x2e2b2b);
      p.setPen(color);
      p.setFont(opt.font);
      p.drawText(QRect(30, 0, 30, 22), Qt::AlignLeft | Qt::AlignVCenter, media->data.at(TRACK::num).toString());
      
      //! paint track name
      QFontMetrics metric( p.font());
      const QString title_elided = metric.elidedText ( media->data.at(TRACK::title).toString(), Qt::ElideRight, title_width);
      p.drawText(QRect(60, 0, title_width, 22), Qt::AlignLeft | Qt::AlignVCenter,title_elided);

      //! paint album name
      const QString album_elided = metric.elidedText ( media->data.at(TRACK::album).toString(), Qt::ElideRight, title_width);
      p.drawText(QRect(60+title_width, 0, title_width, 22), Qt::AlignLeft | Qt::AlignVCenter,album_elided);

      //! paint track duration
      const QString duree_elided = metric.elidedText ( MEDIA::durationTextFromInt(media->data.at(TRACK::duration).toInt()), Qt::ElideRight, 50);
      p.drawText(QRect(60+title_width*2, 0, 50, 22), Qt::AlignRight | Qt::AlignVCenter,duree_elided);

      
      //! paint activated item
      if(media->data.at(TRACK::isPlaying).toBool())
        p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      else if(media->data.at(TRACK::isBroken).toBool())
        p.drawPixmap(0, 0, QPixmap(":/images/media-broken-18x18.png"));

      //! paint track rating
      RatingPainter rating_painter;
      rating_painter.Paint(&p, QRect(title_width*2 + 115, 0, 80, 22),media->data.at(TRACK::rating).toFloat());
      
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);   
}


/*
********************************************************************************
*                                                                              *
*    Class TrackGraphicItem_v3                                                 *
*      -> used in HISTORY VIEW                                                 *
*      -> only draw track or stream name                                       *
*      -> add different color depending on sql id                              *
********************************************************************************
*/ 
TrackGraphicItem_v3::TrackGraphicItem_v3()
{
    _width = 510;
}

QRectF TrackGraphicItem_v3::boundingRect() const
{
    if(_width < 510)
      return QRectF(0, 0, 510, 22);
    
    return QRectF(0, 0, _width, 22);
}

void TrackGraphicItem_v3::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   bool isTrack       = (media->type == TYPE_TRACK) ? true : false;
   bool isPlaying     = isTrack ? media->data.at(TRACK::isPlaying).toBool() : media->data.at(STREAM::isPlaying).toBool();
   bool isBroken      = isTrack ? media->data.at(TRACK::isBroken).toBool() :  media->data.at(STREAM::isBroken).toBool();
   
   const int width       = _width < 530 ? 530 : _width;
   const int title_width = _width < 530 ? 150 : (_width - 150)/3;
   
   QPixmap pixTemp(QSize(width,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect();
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      //! TRACK in collection
      if(media->data.at(TRACK::id) != -1 ) {
        p.setPen(QColor(0x2e2b2b));
        p.setFont(opt.font);
        QFontMetrics metric( p.font());
      
        //! paint artist name
        const QString artist_elided = metric.elidedText ( media->data.at(TRACK::artist).toString(), Qt::ElideRight, title_width);
        p.drawText(QRect(30, 0, title_width-50, 22), Qt::AlignLeft | Qt::AlignVCenter,artist_elided);

        //! paint album name
        const QString album_elided = metric.elidedText ( media->data.at(TRACK::album).toString(), Qt::ElideRight, title_width);
        p.drawText(QRect(60+title_width-50, 0, title_width, 22), Qt::AlignLeft | Qt::AlignVCenter,album_elided);

        //! paint track name
        const QString title_elided = metric.elidedText ( media->data.at(TRACK::title).toString(), Qt::ElideRight, title_width);
        p.drawText(QRect(60+title_width*2-50, 0, title_width, 22), Qt::AlignLeft | Qt::AlignVCenter,title_elided);

        //! paint track duration
        const QString duree_elided = metric.elidedText ( MEDIA::durationTextFromInt(media->data.at(TRACK::duration).toInt()), Qt::ElideRight, 50);
        p.drawText(QRect(60+title_width*3-50, 0, 50, 22), Qt::AlignRight | Qt::AlignVCenter,duree_elided);

        //! paint track rating
        RatingPainter rating_painter;
        rating_painter.Paint(&p, QRect(title_width*3-50 + 115, 0, 80, 22),media->data.at(TRACK::rating).toFloat());
      }
      else {
        p.setPen( QColor(0x707070) );
        p.setFont(opt.font);

        QFontMetrics metric( p.font());
        const QString name_elided = metric.elidedText ( media->data.at(TRACK::url).toString(), Qt::ElideRight, width -20);
        p.drawText(QRect(30, 0,  width -20, 22), Qt::AlignLeft | Qt::AlignVCenter, name_elided);
      }
      
      //! paint activated item
      if(isPlaying)
        p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      else if(isBroken)
        p.drawPixmap(1, 1, QPixmap(":/images/media-broken-18x18.png"));
      else if (!isTrack)
        p.drawPixmap(1, 1, QPixmap(":/images/media-url-18x18.png"));
      
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);   
}


/*
********************************************************************************
*                                                                              *
*    PlaylistGraphicItem                                                       *
*                                                                              * 
********************************************************************************
*/
PlaylistGraphicItem::PlaylistGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, true);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_IconViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   opt.decorationPosition = QStyleOptionViewItem::Top;
   opt.displayAlignment = Qt::AlignCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
}

QRectF PlaylistGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}


void PlaylistGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data
   const QString name    = media->data.at(PLAYLIST::name).toString();
   const QString icon    = media->data.at(PLAYLIST::icon).toString();
   
   QPixmap pixTemp(QSize(140,150));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);

      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect().adjusted(0,0,0,0);
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
   
      QColor color = QColor(0x2e2b2b);
      p.setPen(color);
      p.setFont(opt.font);

      //! paint media playlist mime
      p.drawPixmap(opt.rect.adjusted(15,15,-15,-25),QPixmap(icon));

      //! paint playlist date
      if( media->data.at(PLAYLIST::type).toInt() == T_FILE || 
          media->data.at(PLAYLIST::type).toInt() == T_DATABASE) 
      {
        QDateTime date        =  QDateTime::fromTime_t(media->data.at(PLAYLIST::date).toUInt());
        p.drawText(boundingRect().toRect().adjusted(0, 0, 0, -120), Qt::AlignCenter, date.toString("dd.MM.yyyy"));
      }
      
      //! paint playlist name
      p.setFont(opt.font);
      QFontMetrics fmetric(p.font());
      //QRect fontrect = fmetric.boundingRect (QRect(0, 120, 140, 30),  Qt::AlignCenter | Qt::TextWrapAnywhere, name, 0, 0 );
      const QString elided_p_name = fmetric.elidedText ( name, Qt::ElideMiddle, 260, Qt::TextWrapAnywhere);   

      p.drawText(QRect(0, 120, 140, 30), Qt::AlignTop | Qt::AlignCenter | Qt::TextWrapAnywhere, elided_p_name);
      
      //! paint activated item
      if(media->data.at(PLAYLIST::isPlaying).toBool())
        p.drawPixmap(112, 20, QPixmap(":/images/media-playing.png"));

      //! paint favorites item
      if(media->data.at(PLAYLIST::isFavorite).toBool())  
        p.drawPixmap(114, 50, QPixmap(":/images/favorites-18x18.png"));

      //! paint media playlist type (internal/user)
      if(media->data.at(PLAYLIST::type).toInt() == T_DATABASE) 
        p.drawPixmap(114, 80, QPixmap(":/images/save-18x18.png"));
      
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);  
}


void PlaylistGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event ) 
{
Q_UNUSED(event)
  QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->setData(v); 
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->trigger();  
} 

void PlaylistGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  //Debug::debug() << " ---- AlbumGraphicItem::hoverEnterEvent";
Q_UNUSED(event)
  opt.state |= QStyle::State_MouseOver;

  this->update();
}

void PlaylistGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  //Debug::debug() << " ---- AlbumGraphicItem::hoverLeaveEvent";
  opt.state &= ~QStyle::State_MouseOver;  

  this->update();
}

void PlaylistGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::ClosedHandCursor);  
}

void PlaylistGraphicItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::OpenHandCursor);
}

void PlaylistGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
Q_UNUSED(event)
}



void PlaylistGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();    
    
    //! rechercher dynamique pour les smart playlists
    if(media->data.at(PLAYLIST::type).toInt() == T_SMART)
    {
      mimedata->addMediaItemList( SmartPlaylist::mediaItem( media->data.at(PLAYLIST::rules) ) );
    }
    //! playliste normale -> lecture dans le modele
    else
    { 
     for(int i =0; i < this->media->childCount();i++) {
       if(!PlsFileModel::instance()->isUrlFiltered(this->media->child(i)) ) continue;
       mimedata->addMediaItem(this->media->child(i));
      }
    }
    
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    
    //! set pixmap
    drag->setPixmap( QPixmap(media->data.at(PLAYLIST::icon).toString()) );       
    drag->exec();
    setCursor(Qt::OpenHandCursor);    
}

/*
********************************************************************************
*                                                                              *
*    StreamGraphicItem                                                         *
*                                                                              * 
********************************************************************************
*/
StreamGraphicItem::StreamGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, true);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);  
   
   //! option configuration
   QStyle *style =  QApplication::style();
   opt.widget = 0;
   opt.palette = QApplication::palette();
   opt.font = QApplication::font();
   opt.fontMetrics = QFontMetrics(opt.font);
    
   opt.showDecorationSelected = style->styleHint(QStyle::SH_ItemView_ShowDecorationSelected);
   int pm = style->pixelMetric(QStyle::PM_ListViewIconSize);
    
   opt.decorationSize = QSize(pm, pm);
   
   opt.decorationPosition = QStyleOptionViewItem::Left;
   opt.decorationAlignment = Qt::AlignCenter;
   opt.displayAlignment = Qt::AlignLeft|Qt::AlignVCenter;
   
   opt.locale.setNumberOptions(QLocale::OmitGroupSeparator);
   opt.state |= QStyle::State_Active;
   opt.state |= QStyle::State_Enabled;
   _width = 530;
}

QRectF StreamGraphicItem::boundingRect() const
{
    if(_width < 530)
      return QRectF(0, 0, 530, 22);
    
    return QRectF(0, 0, _width, 22);
}

static int test = 0;
void StreamGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget *widget)
{
Q_UNUSED(option)
   //! get Model data
   if(!media) return;
   const int width       = _width < 530 ? 530 : _width;

   const QString streamname =  media->data.at(STREAM::name).toString();
   
   QPixmap pixTemp(QSize(width,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! Draw frame for State_HasFocus item
      QStyle *style = widget ? widget->style() : QApplication::style();
      opt.rect = boundingRect().toRect().adjusted(0,0,-15,0);
      style->drawControl(QStyle::CE_ItemViewItem, &opt, &p, widget);
      
      //! paint stream name
      QColor color = QColor(0x2e2b2b);
      p.setPen(color);
      p.setFont(opt.font);
      
      QFontMetrics metric( p.font());
      const QString stream_elided = metric.elidedText ( streamname, Qt::ElideRight, width-20);
      p.drawText(QRect(30, 0, width-20, 22), Qt::AlignLeft | Qt::AlignVCenter,stream_elided);
  
      //! paint activated item
      if(media->data.at(STREAM::isPlaying).toBool())
        p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      else if(media->data.at(STREAM::isBroken).toBool())
        p.drawPixmap(2, 1, QPixmap(":/images/media-broken-18x18.png"));
         
      p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
}

void StreamGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->setData(v); 
  (actions->value(BROWSER_ITEM_DOUBLE_CLICK))->trigger();  
} 


void StreamGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  //Debug::debug() << " ---- AlbumGraphicItem::hoverEnterEvent";
  Q_UNUSED(event)
  opt.state |= QStyle::State_MouseOver;

  this->update();
}

void StreamGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  //Debug::debug() << " ---- AlbumGraphicItem::hoverLeaveEvent";
  opt.state &= ~QStyle::State_MouseOver;  

  this->update();
}

void StreamGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::ClosedHandCursor);  
}


void StreamGraphicItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
Q_UNUSED(event)
    setCursor(Qt::OpenHandCursor);
}


void StreamGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  Q_UNUSED(event)
}


void StreamGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    MediaMimeData* mimedata = new MediaMimeData();
    mimedata->addMediaItem(this->media);
  
    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(mimedata);
    drag->exec();
    setCursor(Qt::OpenHandCursor);
}



/*
********************************************************************************
*                                                                              *
*    AlbumGraphicItem_v3                                                       *
*      -> album playcount                                                     *
********************************************************************************
*/
AlbumGraphicItem_v3::AlbumGraphicItem_v3()
{
}

QRectF AlbumGraphicItem_v3::boundingRect() const
{
    return QRectF(0, 0, 150, 175);
}

void AlbumGraphicItem_v3::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get Model data 
   const QString _title     =  media->data.at(ALBUM::name).toString();
   const QString _coverPath =  media->data.at(ALBUM::coverpath).toString();
   const QString _artist    =  media->parent() ? media->parent()->data.at(ARTIST::name).toString() : "";
   const int     playcount  =  media->data.at(ALBUM::playcount).toInt();
      
   double longueur = static_cast<double>(playcount)/_maxPlaycount;
   longueur = longueur*140;
      
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
   
   QPixmap pixTemp(QSize(150,175));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
 
      //! paint cover art
      QPixmap pix;
      if(!QPixmapCache::find(_coverPath, pix)) {
         pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPath);
         if(pix.isNull()) {
            pix = QPixmap(":/images/emptycoverblue110x110.png");
         }
         else {
           QPixmapCache::insert(_coverPath, pix);
         }
      }
      p.drawPixmap(20,27, pix);
      p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
   
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),pixTemp,boundingRect());  


   //! paint bar chart
   draw_playcount(painter, 3, 22, (int)(longueur));
   
   //! paint playcount
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->setPen(QApplication::palette().color(QPalette::Highlight));   
   painter->drawText(QRect(0, 0, 150, 22), Qt::AlignTop | Qt::AlignCenter, QString::number(playcount) );
   
      
   //! paint album title
   painter->setPen(QColor("black"));
   painter->setFont(opt.font); 
   QFontMetrics m1( painter->font());
   const QString elided_album = m1.elidedText ( _title, Qt::ElideRight, 150);
   painter->drawText(QRect (0,135,150, 25), Qt::AlignVCenter | Qt::AlignCenter,elided_album );
   
   //! paint artist name
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->setPen(QColor(0x555555));
   
   QFontMetrics m2( painter->font());
   const QString elided_artist = m2.elidedText ( _artist, Qt::ElideRight, 150);   
   painter->drawText(QRect(0, 157, 150, 18), Qt::AlignTop | Qt::AlignCenter, elided_artist);      
   
   //! paint playing or favorite attibute
   if(media->data.at(ALBUM::isPlaying).toBool())
     painter->drawPixmap(118, 25, QPixmap(":/images/media-playing.png"));
      
   if(media->data.at(ALBUM::isFavorite).toBool())
     painter->drawPixmap(121, 55, QPixmap(":/images/favorites-18x18.png"));
}


/*
********************************************************************************
*                                                                              *
*    AlbumGraphicItem_v4                                                       *
*      -> album rating                                                         *
********************************************************************************
*/
AlbumGraphicItem_v4::AlbumGraphicItem_v4()
{
    // album graphics item with played count bar
}

QRectF AlbumGraphicItem_v4::boundingRect() const
{
    return QRectF(0, 0, 150, 175);
}

void AlbumGraphicItem_v4::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get Model data 
   const QString _title     =  media->data.at(ALBUM::name).toString();
   const QString _coverPath =  media->data.at(ALBUM::coverpath).toString();
   const QString _artist    =  media->parent() ? media->parent()->data.at(ARTIST::name).toString() : "";
      
   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);
   
   QPixmap pixTemp(QSize(150,175));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
 
      //! paint cover art
      QPixmap pix;
      if(!QPixmapCache::find(_coverPath, pix)) {
         pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPath);
         if(pix.isNull()) {
            pix = QPixmap(":/images/emptycoverblue110x110.png");
         }
         else {
           QPixmapCache::insert(_coverPath, pix);
         }
      }
      p.drawPixmap(20,27,pix);
      p.end();
   }
   
   painter->drawPixmap(0, 0, pixTemp);
   
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),pixTemp,boundingRect());  

   //! paint album rating
   RatingPainter rating_painter;
   rating_painter.Paint(painter, QRect(0, 1, 150, 22),media->data.at(ALBUM::rating).toFloat());
   
   //! paint album title
   painter->setPen(QColor("black"));
   painter->setFont(opt.font);
    
   QFontMetrics m1( painter->font());
   const QString elided_album = m1.elidedText ( _title, Qt::ElideRight, 150);
   painter->drawText(QRect (0,135,150, 25), Qt::AlignVCenter | Qt::AlignCenter,elided_album );
   
   //! paint artist name
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->setPen(QColor(0x555555));
   
   QFontMetrics m2( painter->font());
   const QString elided_artist = m2.elidedText ( _artist, Qt::ElideRight, 150);   
   painter->drawText(QRect(0, 157, 150, 18), Qt::AlignTop | Qt::AlignCenter, elided_artist);      
   
   //! paint playing or favorite attibute
   if(media->data.at(ALBUM::isPlaying).toBool())
     painter->drawPixmap(118, 25, QPixmap(":/images/media-playing.png"));
      
   if(media->data.at(ALBUM::isFavorite).toBool())
     painter->drawPixmap(121, 55, QPixmap(":/images/favorites-18x18.png"));
}







/*
********************************************************************************
*                                                                              *
*    ArtistGraphicItem_v2                                                      *
*      -> artist playcount                                                     *
********************************************************************************
*/
ArtistGraphicItem_v2::ArtistGraphicItem_v2()
{
}

QRectF ArtistGraphicItem_v2::boundingRect() const
{
    //ArtistGraphicItem is  QRectF(0, 0, 140, 150);
    return QRectF(0, 0, 150, 175);
}

void ArtistGraphicItem_v2::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   const QString      _name          = media->data.at(ARTIST::name).toString();
   const QStringList  _coverPathList = media->data.at(ARTIST::coverpathlist).toStringList();
   const int     playcount           = media->data.at(ARTIST::playcount).toInt();
      
   double longueur = static_cast<double>(playcount)/_maxPlaycount;
   longueur = longueur*140;

   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);   
     
   QPixmap basePixmap = QPixmap( 150, 175 );
   {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );

     //! affichage des covers
     for (int i = 0; i <_coverPathList.size(); i++) {
         QPixmap pix;
         if(!QPixmapCache::find(_coverPathList[i], pix)) {
           pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPathList[i]);
           if(pix.isNull()) {
             pix = QPixmap(":/images/emptycoverblue110x110.png");
           }
           else {
             QPixmapCache::insert(_coverPathList[i], pix);
           }
        }
 
         pix = pix.scaled(QSize(85,85), Qt::KeepAspectRatio, Qt::SmoothTransformation);
         qreal angle =  15*i;

         QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
         pix = pix.transformed(transform, Qt::SmoothTransformation); 
      
         pt.drawPixmap(boundingRect().toRect().adjusted((150 -pix.width())/2,
                                                        (175 -pix.height())/2 -4,
                                                       -(150 -pix.width())/2, 
                                                       -(175-pix.height())/2 -4)
                                                        , pix);
     }
     pt.end();
   }
   painter->drawPixmap(boundingRect().toRect(), basePixmap);
  
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),basePixmap,boundingRect());  
  
   //! paint bar chart
   draw_playcount(painter, 3, 22, (int)(longueur));
   
   //! paint playcount
   painter->setFont(QFont("Arial", 8, QFont::Bold));
   painter->setPen(QApplication::palette().color(QPalette::Highlight));   
   painter->drawText(QRect(0, 0, 150, 22), Qt::AlignTop | Qt::AlignCenter, QString::number(playcount) );
   
   //! paint artist name
   painter->setPen(QColor(0x2e2b2b));
   painter->setFont(opt.font);
    
   QRect textBox = boundingRect().toRect().adjusted(0, 125, 0, 0);
   QFontMetrics fontmetric(painter->font());
   QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _name, 0, 0 );

   painter->drawText(textBox.adjusted(0, fontrect.height(), 0, 0), Qt::AlignCenter | Qt::TextWordWrap, _name);

   //! paint playing or favorite attibute
   if(media->data.at(ARTIST::isPlaying).toBool())
     painter->drawPixmap(118, 25, QPixmap(":/images/media-playing.png"));
      
   if(media->data.at(ARTIST::isFavorite).toBool())
     painter->drawPixmap(121, 55, QPixmap(":/images/favorites-18x18.png"));
}



/*
********************************************************************************
*                                                                              *
*    ArtistGraphicItem_v3                                                      *
*      -> artist rating                                                        * 
********************************************************************************
*/
ArtistGraphicItem_v3::ArtistGraphicItem_v3()
{

}

QRectF ArtistGraphicItem_v3::boundingRect() const
{
    //ArtistGraphicItem is  QRectF(0, 0, 140, 150);
    return QRectF(0, 0, 150, 175);
}

void ArtistGraphicItem_v3::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget)
{
Q_UNUSED(option)
   //! get data 
   const QString      _name          = media->data.at(ARTIST::name).toString();
   const QStringList  _coverPathList = media->data.at(ARTIST::coverpathlist).toStringList();

   //! Draw frame for State_HasFocus item
   QStyle *style = widget ? widget->style() : QApplication::style();
   opt.rect = boundingRect().toRect().adjusted(0,0,0,0);  
   style->drawControl(QStyle::CE_ItemViewItem, &opt, painter, widget);   
     
   QPixmap basePixmap = QPixmap( 150, 175 );
   {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );

     //! affichage des covers
     for (int i = 0; i <_coverPathList.size(); i++) {
         QPixmap pix;
         if(!QPixmapCache::find(_coverPathList[i], pix)) {
           pix = QPixmap(UTIL::CONFIGDIR + "/albums/" + _coverPathList[i]);
           if(pix.isNull()) {
             pix = QPixmap(":/images/emptycoverblue110x110.png");
           }
           else {
             QPixmapCache::insert(_coverPathList[i], pix);
           }
        }
 
         pix = pix.scaled(QSize(85,85), Qt::KeepAspectRatio, Qt::SmoothTransformation);
         qreal angle =  15*i;

         QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
         pix = pix.transformed(transform, Qt::SmoothTransformation); 
      
         pt.drawPixmap(boundingRect().toRect().adjusted((150 -pix.width())/2,
                                                        (175 -pix.height())/2 -4,
                                                       -(150 -pix.width())/2, 
                                                       -(175-pix.height())/2 -4)
                                                        , pix);
     }
     pt.end();
   }
   painter->drawPixmap(boundingRect().toRect(), basePixmap);
  
   //! manual graphics shadow (Qgraphics Effect cause refresh pb)
   drawShadow(painter,QPointF(0,0),basePixmap,boundingRect());  
  
   //! paint album rating
   RatingPainter rating_painter;
   rating_painter.Paint(painter, QRect(0, 1, 150, 22),media->data.at(ARTIST::rating).toFloat());
     
   //! paint artist name
   painter->setPen(QColor(0x2e2b2b));
   painter->setFont(opt.font);
    
   QRect textBox = boundingRect().toRect().adjusted(0, 125, 0, 0);
   QFontMetrics fontmetric(painter->font());
   QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _name, 0, 0 );

   painter->drawText(textBox.adjusted(0, fontrect.height(), 0, 0), Qt::AlignCenter | Qt::TextWordWrap, _name);

   //! paint playing or favorite attibute
   if(media->data.at(ARTIST::isPlaying).toBool())
     painter->drawPixmap(118, 25, QPixmap(":/images/media-playing.png"));
      
   if(media->data.at(ARTIST::isFavorite).toBool())
     painter->drawPixmap(121, 55, QPixmap(":/images/favorites-18x18.png"));
}

/*
********************************************************************************
*                                                                              *
*    draw_playcount                                                            *
*                                                                              * 
********************************************************************************
*/

void draw_playcount(QPainter* painter, int Xstart,int Ystart, int Xstop)
{
      // Draw a nice gradient first
      QColor line_color(QApplication::palette().color(QPalette::Highlight));
      QColor shadow_color(line_color.lighter(140));
      QColor shadow_fadeout_color(shadow_color);
      shadow_color.setAlpha(255);
      shadow_fadeout_color.setAlpha(0);

      QLinearGradient gradient(QPoint(Xstart, Ystart - 5),QPoint(Xstart, Ystart + 5));
      gradient.setColorAt(0.0, shadow_fadeout_color);
      gradient.setColorAt(0.5, shadow_color);
      gradient.setColorAt(1.0, shadow_fadeout_color);
      QPen gradient_pen(QBrush(gradient), 4);
      painter->setPen(gradient_pen);
      painter->drawLine(QPoint(Xstart, Ystart),QPoint(Xstop, Ystart));
        
      // Now draw the line on top
      QPen line_pen(line_color, 2);
      painter->setPen(line_pen);
      painter->drawLine(QPoint(Xstart, Ystart),QPoint(Xstop, Ystart));
}