/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "graphicitem_menu.h"

#include "graphicitem.h"
#include "collectionmodel.h"
#include "plsfilemodel.h"
#include "streammodel.h"

/*
********************************************************************************
*                                                                              *
*    Class MenuTrack                                                           *
*                                                                              *
********************************************************************************
*/   
class MenuTrack : public QWidget
{
public:
    MenuTrack(QWidget* parent,const QString& text) : QWidget(parent)
    {
       m_text = text ;
       m_parent = parent;
       //resize( sizeHint() );
    }
    
protected:
    void paintEvent(QPaintEvent *event);
    QSize sizeHint() const;

private:
    QString m_text;
    QWidget *m_parent;
};

QSize MenuTrack::sizeHint() const
{
    QFontMetrics fm( QFont("Segoe UI", 9, QFont::Normal) );
    const int x = fm.boundingRect(m_text).width() + 12 /* for icon */ + 20;
    const int y = fm.height();
    return QSize(x, y);       
}


void MenuTrack::paintEvent(QPaintEvent *event)
{
    QPainter p(this);
    p.setFont( QFont("Segoe UI", 9, QFont::Normal) );
      
    QRect rect = event->rect();
      
    QIcon icon = QIcon(":/images/track-48x48.png");
    icon.paint(&p, 1, rect.top()+3, 12, 12, Qt::AlignVCenter | Qt::AlignLeft, QIcon::Normal);      
    p.drawText(20,rect.top(),rect.width(),rect.height(),Qt::AlignVCenter | Qt::AlignLeft, m_text);

    p.end();
}


/*
********************************************************************************
*                                                                              *
*    Class GraphicsItemMenu                                                    *
*                                                                              *
********************************************************************************
*/   
GraphicsItemMenu::GraphicsItemMenu(QWidget *parent) : QMenu(parent)
{
    this->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);

    m_graphicitem = 0;
    
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::WindowText, Qt::black);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor ( QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Base, Qt::darkGray);
    palette.setColor ( QPalette::Disabled, QPalette::Text, Qt::darkGray);    

    this->setAttribute(Qt::WA_NoBackground, true);
    this->setPalette(palette);
    this->setAutoFillBackground(true);
    
    
    //! creates menu actions
    m_map_actions.insert(ALBUM_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play Album"), 0));
    m_map_actions.insert(ALBUM_LOAD_COVER,    new QAction(QIcon(":/images/album.png"), tr("Load Cover Art from File"), 0));
    m_map_actions.insert(ALBUM_LASTFM_COVER,  new QAction(QIcon(":/images/lastfm.png"), tr("Load Cover Art from LastFm"), 0));
    m_map_actions.insert(ALBUM_REMOVE_COVER,  new QAction(QIcon::fromTheme("edit-delete"), tr("Remove Cover Art"), 0));
    m_map_actions.insert(ALBUM_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit album"), 0));
    m_map_actions.insert(ALBUM_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(ALBUM_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(ALBUM_FAVORITE,      new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add Album to favorites"), 0));

    m_map_actions.insert(GENRE_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play Album"), 0));
    m_map_actions.insert(GENRE_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(GENRE_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(GENRE_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit album"), 0));

    m_map_actions.insert(ARTIST_PLAY,         new QAction(QIcon(":/images/media-play.png"), tr("Play Artist"), 0));
    m_map_actions.insert(ARTIST_EDIT,         new QAction(QIcon(":/images/edit_info.png"), tr("Edit artist"), 0));
    m_map_actions.insert(ARTIST_QUEUE_END,    new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(ARTIST_QUEUE_NEW,    new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(ARTIST_FAVORITE,     new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add"), 0));

    m_map_actions.insert(PLAYLIST_PLAY,       new QAction(QIcon(":/images/media-play.png"), tr("Play Playlist"), 0));
    m_map_actions.insert(PLAYLIST_EDIT,       new QAction(QIcon(":/images/edit_info.png"), tr("Edit playlist"), 0));
    m_map_actions.insert(PLAYLIST_QUEUE_END,  new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(PLAYLIST_QUEUE_NEW,  new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(PLAYLIST_FAVORITE,   new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add Playlist to favorites"), 0));
    m_map_actions.insert(PLAYLIST_REMOVE,     new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove playlist from disk"), 0));

    m_map_actions.insert(TRACK_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play track"), 0));
    m_map_actions.insert(TRACK_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit track"), 0));
    m_map_actions.insert(TRACK_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(TRACK_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));

    m_map_actions.insert(STREAM_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play stream"), 0));
    m_map_actions.insert(STREAM_EDIT,          new QAction(QIcon(":/images/edit_info.png"), tr("Edit stream"), 0));
    m_map_actions.insert(STREAM_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
    m_map_actions.insert(STREAM_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
    m_map_actions.insert(STREAM_FAVORITE,      new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));
    
    // connect each actions
    foreach (QAction * action, m_map_actions.values() ) {
      QObject::connect(action, SIGNAL(triggered()), this, SLOT(slot_actionsTriggered()));
    }
}

// public
void GraphicsItemMenu::setGraphicItem(QGraphicsItem* item)
{
    m_graphicitem = item;
    updateMenuUi();
}


// private
void GraphicsItemMenu::updateMenuUi()
{
  // delete action_widget
  this->clear();
  if(!m_graphicitem) return;
    
  /*-------------------------------------------------*/
  /* AlbumType                                       */
  /* ------------------------------------------------*/    
  if( m_graphicitem->type() == GraphicsItem::AlbumType )
  { 
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(m_graphicitem);

    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(1);
    main_box->setContentsMargins(4,4,4,4);
    
    QWidget* main_widget = new QWidget();
    main_widget->setLayout(main_box);

    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
        
    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(ALBUM_PLAY) );
    tool_bar->addAction( m_map_actions.value(ALBUM_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(ALBUM_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ALBUM_LOAD_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_LASTFM_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_REMOVE_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_EDIT) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ALBUM_FAVORITE) );

    QFrame* separator = new QFrame(main_widget);
    separator->setFrameShape(QFrame::HLine);
    separator->setFrameShadow(QFrame::Sunken);    

    // populate layout
    main_box->addWidget(new QLabel(item->media->data.at(ALBUM::name).toString()) );
    main_box->addWidget(tool_bar);
    main_box->addWidget(separator);
    
    // update data from album media item
    QList<MediaItemPtr> list = CollectionModel::instance()->getItemChildrenTracks(item->media);
    foreach(MediaItemPtr m, list) {
        main_box->addWidget(new MenuTrack(0,m->data.at(TRACK::title).toString()) );
    }    

    this->addAction( action_widget );
    
    if(CollectionModel::instance()->itemIsFavorite(item->media))
      m_map_actions.value(ALBUM_FAVORITE)->setText( tr("Remove Album from Favorites") );
    else
      m_map_actions.value(ALBUM_FAVORITE)->setText( tr("Add Album to Favorites") );
  }
  /*-------------------------------------------------*/
  /* AlbumGenreType                                  */
  /* ------------------------------------------------*/
  else if( m_graphicitem->type() == GraphicsItem::AlbumGenreType )
  {
    AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(m_graphicitem);
    
    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(1);
    main_box->setContentsMargins(4,4,4,4);
    
    QWidget* main_widget = new QWidget();
    main_widget->setLayout(main_box);
    
    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );
    
    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(GENRE_PLAY) );
    tool_bar->addAction( m_map_actions.value(GENRE_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(GENRE_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ALBUM_LOAD_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_LASTFM_COVER) );
    tool_bar->addAction( m_map_actions.value(ALBUM_REMOVE_COVER) );
    tool_bar->addAction( m_map_actions.value(GENRE_EDIT) );

    QFrame* separator = new QFrame(main_widget);
    separator->setFrameShape(QFrame::HLine);
    separator->setFrameShadow(QFrame::Sunken);

    // populate layout
    main_box->addWidget(new QLabel(item->media->data.at(ALBUM::name).toString()) );
    main_box->addWidget(tool_bar);
    main_box->addWidget(separator);

    QList<MediaItemPtr> list = CollectionModel::instance()->getAlbumChildrenTracksGenre(item->media,item->_genre);
    foreach(MediaItemPtr m, list) {
        //main_box->addWidget(new QLabel(m->data.at(TRACK::title).toString()) );
        main_box->addWidget(new MenuTrack(0,m->data.at(TRACK::title).toString()) );
    }

    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* ArtistType                                      */
  /* ------------------------------------------------*/  
  else if( m_graphicitem->type() == GraphicsItem::ArtistType )
  {
    ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(m_graphicitem);
    
    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(1);
    main_box->setContentsMargins(4,4,4,4);
    
    QWidget* main_widget = new QWidget();
    main_widget->setLayout(main_box);
   
    QWidgetAction * action_widget = new QWidgetAction( this );
    action_widget->setDefaultWidget( main_widget );    
    
    
    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(ARTIST_PLAY) );
    tool_bar->addAction( m_map_actions.value(ARTIST_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(ARTIST_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ARTIST_EDIT) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(ARTIST_FAVORITE) );
    
    /*QFrame* separator = new QFrame(main_widget);
    separator->setFrameShape(QFrame::HLine);
    separator->setFrameShadow(QFrame::Sunken);*/

    // populate layout
    main_box->addWidget(new QLabel(item->media->data.at(ARTIST::name).toString()) );
    main_box->addWidget(tool_bar);
    //main_box->addWidget(separator);

    if(CollectionModel::instance()->itemIsFavorite(item->media))
      m_map_actions.value(ARTIST_FAVORITE)->setText( tr("Remove Artist from Favorites") );
    else
      m_map_actions.value(ARTIST_FAVORITE)->setText( tr("Add Artist to Favorites") );

    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* TrackType                                       */
  /* ------------------------------------------------*/    
  else if( m_graphicitem->type() == GraphicsItem::TrackType )
  {
    //TrackGraphicItem *item = static_cast<TrackGraphicItem*>(m_graphicitem);
    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(1);
    main_box->setContentsMargins(4,4,4,4);
    
    QWidget* main_widget = new QWidget();
    main_widget->setLayout(main_box);
   
    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );    
        
    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(TRACK_PLAY) );
    
    
    TrackGraphicItem *item = static_cast<TrackGraphicItem*>(m_graphicitem);
    if( item->media->data.at(TRACK::id).toInt() != -1)
      tool_bar->addAction( m_map_actions.value(TRACK_EDIT) );
    
    tool_bar->addAction( m_map_actions.value(TRACK_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(TRACK_QUEUE_NEW) );
    
    main_box->addWidget(tool_bar);

    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* PlaylistType                                    */
  /* ------------------------------------------------*/     
  else if( m_graphicitem->type() == GraphicsItem::PlaylistType )
  {
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(m_graphicitem);

    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(1);
    main_box->setContentsMargins(4,4,4,4);
    
    QWidget* main_widget = new QWidget();
    main_widget->setLayout(main_box);
   
    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );    
    
    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_PLAY) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_QUEUE_NEW) );
    tool_bar->addSeparator();
    tool_bar->addAction( m_map_actions.value(PLAYLIST_REMOVE) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_EDIT) );
    tool_bar->addAction( m_map_actions.value(PLAYLIST_FAVORITE) );
    
    /*QFrame* separator = new QFrame(main_widget);
    separator->setFrameShape(QFrame::HLine);
    separator->setFrameShadow(QFrame::Sunken);*/

    // populate layout    
    main_box->addWidget(new QLabel(item->media->data.at(PLAYLIST::name).toString()) );
    main_box->addWidget(tool_bar);
    //main_box->addWidget(separator);

    if(PlsFileModel::instance()->itemIsFavorite(item->media))
      m_map_actions.value(PLAYLIST_FAVORITE)->setText( tr("Remove Playlist from Favorites") );
    else
      m_map_actions.value(PLAYLIST_FAVORITE)->setText( tr("Add Playlist to Favorites") );

    this->addAction( action_widget );
  }
  /*-------------------------------------------------*/
  /* StreamType                                       */
  /* ------------------------------------------------*/    
  else if( m_graphicitem->type() == GraphicsItem::StreamType )
  {
    StreamGraphicItem *item = static_cast<StreamGraphicItem*>(m_graphicitem);
    
    QVBoxLayout * main_box = new QVBoxLayout();
    main_box->setSpacing(1);
    main_box->setContentsMargins(4,4,4,4);
    
    QWidget* main_widget = new QWidget();
    main_widget->setLayout(main_box);
   
    QWidgetAction * action_widget = new QWidgetAction(this);
    action_widget->setDefaultWidget( main_widget );    
        
    QToolBar *tool_bar = new QToolBar(main_widget);
    tool_bar->setIconSize( QSize( 16, 16 ) );
    tool_bar->addAction( m_map_actions.value(STREAM_PLAY) );
    //tool_bar->addAction( m_map_actions.value(STREAM_EDIT) );
    tool_bar->addAction( m_map_actions.value(STREAM_QUEUE_END) );
    tool_bar->addAction( m_map_actions.value(STREAM_QUEUE_NEW) );
    tool_bar->addAction( m_map_actions.value(STREAM_FAVORITE) );
    
    if(StreamModel::instance()->itemIsFavorite(item->media))
      m_map_actions.value(STREAM_FAVORITE)->setText( tr("Remove from favorites") );
    else
      m_map_actions.value(STREAM_FAVORITE)->setText( tr("Add to favorites") );

    main_box->addWidget(tool_bar);
    //main_box->addWidget(separator);

    this->addAction( action_widget );
  }  
}



// private slots
void GraphicsItemMenu::slot_actionsTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if(action) {
      emit menu_action_triggered( (ENUM_ACTION_ITEM_MENU)m_map_actions.key(action) );
      this->hide();
    }
}
