/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _GRAPHIC_VIEW_WIDGET_H_
#define _GRAPHIC_VIEW_WIDGET_H_

#include "viewmode.h"

#include <QGraphicsView>
#include <QHash>
#include <QString>

class CollectionScene;
class StreamScene;
class ContextScene;
class SettingsScene;
/*
********************************************************************************
*                                                                              *
*    GraphicViewWidget                                                         *
*                                                                              *
********************************************************************************
*/
class GraphicViewWidget : public QGraphicsView
{
  Q_OBJECT
public: 
    GraphicViewWidget(QWidget *parent);
    void setCollectionScene(CollectionScene *scene);
    void setStreamScene(StreamScene *scene);
    void setContextScene(ContextScene *scene);
    void setSettingsScene(SettingsScene *scene);
    
    void setSearch(const QString& search);
    MAIN::ViewMode mode() {return _viewMode;}
    
    void set_scroll_to(QPoint p);

public slots:
    void updateView();

protected:
    void resizeEvent ( QResizeEvent * event );
  
private slots:
    void checkSlider(int);
    void switchView(QAction* action=0);
    void updateSearch(const QString& );
    void updateStatusWidget();
    void navigateGoUp();
    void navigateGoDown();    

private :
    /* scene */
    CollectionScene      *_collectionScene;
    StreamScene          *_streamScene;
    ContextScene         *_contextScene;
    SettingsScene        *_settingsScene;
    
    /* mode */
    MAIN::ViewMode       _viewMode;
    
    /* scroll position */
    QHash<MAIN::ViewMode, int>  m_scrollPosition;
};

#endif // _GRAPHIC_VIEW_WIDGET_H_
