/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "StreamModel/streammodel.h"
#include "MediaItem/mediaitem.h"
#include "debug.h"


StreamModel* StreamModel::INSTANCE = 0;

/*
********************************************************************************
*                                                                              *
*    Class StreamModel                                                         *
*                                                                              *
********************************************************************************
*/
StreamModel::StreamModel(QObject *parent) : QObject(parent) 
{
    Debug::debug() << "StreamModel -> creation";
    INSTANCE          = this;
    m_filter_pattern  = "";
}

void StreamModel::clear()
{
    foreach(MediaItemPtr mi, m_listStream)
      mi.reset();
  
    m_listStream.clear();
}

//! ------------------ StreamModel::updateListeStream --------------------------
void StreamModel::updateListeStream(const QList<MediaItemPtr> &liste)
{
    //Debug::debug() << "StreamModel -> updateListeStream";
    this->clear();
    if (!liste.empty()) {
      m_listStream.append(liste);
    }
}


//!------------StreamModel::getStreamItem ---------------------------------------
MediaItemPtr StreamModel::getStreamItem(int row) 
{
    if (row < 0 || row >= m_listStream.size())
      return MediaItemPtr(0);

    return m_listStream.at(row);
}

//! --------- playing item method ----------------------------------------------
void StreamModel::updatePlayingItem(const QString& urlName)
{
    Debug::debug() << "StreamModel -> updatePlayingItem" << urlName;

    for ( int i = 0; i < m_listStream.size(); i++ )
    {
      MediaItemPtr mi = m_listStream.at(i);
      if (urlName ==  MEDIA::urlForMediaItem(mi)) 
      {
        mi->data[STREAM::isPlaying]         = true;
        emit signalPlayingStatusChanged();
        break;
      }
    }
}

void StreamModel::stopPlayingItem()
{
    for ( int i = 0; i < m_listStream.size(); i++ )
    {
      MediaItemPtr mi = m_listStream.at(i);
      if (mi->data.at(STREAM::isPlaying).toBool()) {
        mi->data[STREAM::isPlaying] = false;
        emit signalPlayingStatusChanged();
        break;
      }
    }
}  
  
//! --------- Filtering method -------------------------------------------------
bool StreamModel::isStreamFiltered(const int row)
{
    MediaItemPtr mi = getStreamItem(row);
    if (mi->data.at(STREAM::name).toString().contains(m_filter_pattern, Qt::CaseInsensitive) ||
        mi->data.at(STREAM::categorie).toString().contains(m_filter_pattern, Qt::CaseInsensitive)
        )
        return true;
  
    return false;
}


//! --------- broken item method ----------------------------------------------
void StreamModel::updateBrokenItem(const QString& urlName)
{
    Debug::debug() << "StreamModel -> updateBrokenItem" << urlName;

    for ( int i = 0; i < m_listStream.size(); i++ )
    {
      MediaItemPtr mi = m_listStream.at(i);
      if (urlName ==  mi->data.at(STREAM::url).toString()) 
      {
        mi->data[STREAM::isBroken]         = true;
        emit signalPlayingStatusChanged();
        break;
      }
    }
}


//! ----------------------- Favorite status method -----------------------------
bool StreamModel::itemIsFavorite(const MediaItemPtr item) 
{
    switch (item->type) {
      case TYPE_STREAM : return item->data.at(STREAM::isFavorite).toBool();break;
      default: break;
    }
    return false;
}


void StreamModel::setItemIsFavorite(const MediaItemPtr item, bool isfavorite) 
{
    switch (item->type) {
      case TYPE_STREAM : item->data[STREAM::isFavorite] = isfavorite;break;
      default: break;
    }
}